--- Guilder: Responder Module
-- This module allows the user to set a response message to specific groups
-- @class file
-- @name Modules/responder.lua
-- @release 1.0.0

if select(6, GetAddOnInfo("Guilder_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local Guilder = _G.Guilder
if not Guilder then
	error("Guilder_BusyResponder requires Guilder")
end

--[[ *** LOAD LIBRARIES *** ]]--

local Guilder_BusyResponder = Guilder:NewModule("Busy Responder")
local L = Guilder.L
Guilder_BusyResponder.modName = L["Busy Responder"]

--[[ *** DEFAULT SETTINGS *** ]]--

local defaults = {
	profile = {
		message = L["BUSY_RESPONSE"],
		officer = true,
		guild = true,
		party = true,
		raid = true,
		friend = true,
	},
}

--[[ *** CONFIGURATION MENU *** ]]--

local options = {
	RespondToOfficers = {
		type = "toggle",
		name = L["Respond to Guild Officers"],
		desc = L["Choose whether the responder should respond to officers of your guild"],
		width = "full",
		order = 20,
		get = function() return Guilder_BusyResponder.db.profile.officer end,
		set = function(info, v) Guilder_BusyResponder.db.profile.officer = v end,
	},
	RespondToMembers = {
		type = "toggle",
		name = L["Respond to Guild Members"],
		desc = L["Choose whether the responder should respond to members of your guild"],
		width = "full",
		order = 21,
		get = function() return Guilder_BusyResponder.db.profile.guild end,
		set = function(info, v) Guilder_BusyResponder.db.profile.guild = v end,
	},
	RespondToParty = {
		type = "toggle",
		name = L["Respond to Party Members"],
		desc = L["Choose whether the responder should respond to members of your party"],
		width = "full",
		order = 22,
		get = function() return Guilder_BusyResponder.db.profile.party end,
		set = function(info, v) Guilder_BusyResponder.db.profile.party = v end,
	},
	RespondToRaid = {
		type = "toggle",
		name = L["Respond to Raid Members"],
		desc = L["Choose whether the responder should respond to members of your raid"],
		width = "full",
		order = 23,
		get = function() return Guilder_BusyResponder.db.profile.raid end,
		set = function(info, v) Guilder_BusyResponder.db.profile.raid = v end,
	},
	RespondToFriends = {
		type = "toggle",
		name = L["Respond to Friends"],
		desc = L["Choose whether the responder should respond to members of your friends-list"],
		width = "full",
		order = 24,
		get = function() return Guilder_BusyResponder.db.profile.friend end,
		set = function(info, v) Guilder_BusyResponder.db.profile.friend = v end,
	},
	CustomiseMessage = {
		type = "input",
		name = L["Customise Message"],
		desc = L["Customise the message sent by the responder"],
		multiline = true,
		width = "full",
		order = 25,
		get = function() return Guilder_BusyResponder.db.profile.message end,
		set = function(info, v) Guilder_BusyResponder.db.profile.message = v end,
	},
	ResetMessage = {
		type = "execute",
		name = L["Reset Message"],
		desc = L["Reset the message to the default message originally supplied"],
		order = 26,
		func = function() Guilder_BusyResponder.db.profile.message = L["BUSY_RESPONSE"] end,
	},
	TestMessage = {
		type = "execute",
		name = L["Test Message"],
		desc = L["Test the BusyResponder message by whispering it to yourself"],
		order = 27,
		func = function()
			Guilder:SendMessage("BusyResponder - "..Guilder_BusyResponder.db.profile.message, "WHISPER", UnitName("player"))
		end,
	},
}

--[[ *** MODULE FUNCTIONS *** ]]--

function Guilder_BusyResponder:OnInitialize()
	if Guilder.db.profile.modules["Busy Responder"] == nil then Guilder.db.profile.modules["Busy Responder"] = false end
	self.db = Guilder.db:RegisterNamespace("Busy Responder", defaults)
end

function Guilder_BusyResponder:OnEnable()
end

function Guilder_BusyResponder:OnDisable()
end

function Guilder_BusyResponder:Info()
	return L["Auto-respond to whispers with a busy message with this tool"]
end

function Guilder_BusyResponder:DoCommand(sender, msg)
end

function Guilder_BusyResponder:GetCommand()
	return nil
end

function Guilder_BusyResponder:GetHelp()
	return nil
end

function Guilder_BusyResponder:GetOptions()
	return options
end

--[[ *** BUSY RESPONDER FUNCTIONS *** ]]--

--- This function sends a response to the sender of a whispered message if necessary
--- User can choose to respond to any of the following:
--- o  someone in our guild.
--- o  someone an officer of our guild.
--- o  someone in our party.
--- o  someone in our raid.
--- o  someone on our friends-list.
-- @param sender name of the person who whispered us
-- @param msg the original message receiver (used to filter out messages starting with "[guilder]")
-- @usage internal use only
-- @returns nil
function Guilder_BusyResponder:SendBusyResponder(sender, msg)
	-- perform the guild-member exclusion check
	if LG:HasMember(sender) and Guilder_BusyResponder.db.profile.guild == false then
		return
	end
	-- perform the officer-member exclusion check
	if LG:HasMember(sender) and Guilder:IsOfficer(sender) and Guilder_BusyResponder.db.profile.officer == false then
		return
	end
	-- perform the party-member exclusion check
	if UnitInParty(sender) and Guilder_BusyResponder.db.profile.party == false then
		return
	end
	-- perform the raid-member exclusion check
	if UnitInRaid(sender) and Guilder_BusyResponder.db.profile.raid == false then
		return
	end
	-- perform the friends-list exclusion check
	if Guilder:UnitInFriends(sender) and Guilder_BusyResponder.db.profile.friend == false then
		return
	end
	-- perform the "responding to a [BusyResponder] message" check
	local brMessageTest = string.find(msg:lower(), "^%[guilder%] busyresponder")
	if brMessageTest then
		return
	end
	-- send the response message
	Guilder:SendMessage("BusyResponder - "..Guilder_BusyResponder.db.profile.message, "WHISPER", sender)
end
